# -*- python -*-
# $RCSfile: progressbarGUI.py,v $
# $Revision: 1.13 $
# $Author: langer $
# $Date: 2004/07/08 14:11:11 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.common.IO import progressbar
from common import guitop
from common.IO import reporter
from common.IO import threadmanager
from common.IO.GUI import activityViewer
import gtk

#########################################################################
#########################################################################
## The following is the Graphic progress bar class and sends messages to
## GTK.
#########################################################################
#########################################################################


class GUIProgressBar (progressbar.ProgressBarRepresentation):
    def __init__(self, progressBar):
        progressbar.ProgressBarRepresentation.__init__(self, progressBar)
                
        self.timer = 200 ## in miliseconds
        self.success = 0
        ## button defaults
        self.orientation = gtk.PROGRESS_LEFT_TO_RIGHT
        self.style = gtk.PROGRESS_CONTINUOUS
        ## the bar is continuous by default.
        
        
        self.progressbox = None
        self.gtkprogressbar = None
        self.label_bottom = None
        self.click_counter = 0
        ## Here, the bar is created and gtk becomes aware of two functions which
        ## are used to update the bar/GUI
        self.create_bar()
        self.attachGUIcallbacks()

    def get_name(self):
        return self.progressbar.get_name()
    
    def progress_discrete(self):
        self.style = gtk.PROGRESS_DISCRETE
        
    
    def create_bar(self):
        ## Here we create the actual bar graphic representation
        self.destroy_bar = 0
        if  self.progressbox is None:
            self.progressbox = gtk.GtkVBox(gtk.FALSE, 10)

            ## Create progress line
            self.progressline = gtk.GtkHBox(gtk.FALSE, 0)
                
            ## Create the progress bar
            self.gtkprogressbar = gtk.GtkProgressBar()
            self.gtkprogressbar.set_orientation(self.orientation)
            if self.type == "discrete":
                self.progress_discrete() 
            elif self.type == "active":
                self.back_and_forth_state(gtk.TRUE)
                ## self.back_and_forth_state()
                
            
            self.gtkprogressbar.set_bar_style(self.style)
            self.progressline.pack_start(self.gtkprogressbar, gtk.TRUE,
                                         gtk.TRUE, 10)
            self.progressbox.pack_start(self.progressline, gtk.FALSE,
                                        gtk.FALSE, 0)
            self.gtkprogressbar.show()
            
            ## End of progress bar creation

            
            ## Stop button lives here
            self.stopbutton = gtk.GtkButton("Stop")
            self.stopbutton.connect("clicked", self.update_button_state)
            self.progressline.pack_start(self.stopbutton, gtk.TRUE, gtk.TRUE, 0)
            self.stopbutton.show()
            self.progressline.show()
            self.name = self.get_name() 
            ## Create the bottom label, if any
            if self.name and not self.label_bottom:
                self.scrllwd = gtk.GtkScrolledWindow(None,None)
                self.scrllwd.set_policy(gtk.POLICY_AUTOMATIC,
                                        gtk.POLICY_AUTOMATIC)
                self.progressbox.add(self.scrllwd) 
                self.scrllwd.show()
                self.label_bottom = gtk.GtkText()
                self.label_bottom.set_editable(0)
                self.label_bottom.set_word_wrap(1)
                
                
                self.scrllwd.add(self.label_bottom)
                self.label_bottom.insert(None, None, None, self.name)
                self.label_bottom.show()

            

    def back_and_forth_state(self, val):
        self.gtkprogressbar.set_activity_mode(val)
        self.gtkprogressbar.set_activity_step(35)
    def schedule_gtk_removal(self):
        ## print "schedule bar removal"
        self.destroy_bar = 1
        switchboard.notify("dismiss gtkbar",self)

    ## def update_text(self, message = "  "):
    ##    self.name = message
        
    def set_text(self, text = None): ## this updates the text
        if text :
            self.label_bottom.insert(None, None, None,text)
        


    def destroy(self):
        if self.destroy_bar :
            ## print "destroying bar"
            self.progressbox.destroy()
            gtk.timeout_remove(self.bar_tag)
            activityViewer.sensitize()
##            switchboard.notify("sensitize activity viewer")
            switchboard.notify("remove worker", self.progressbar.Worker)
            self.progressbar.unregisterWorker() ## remove worker reference from pbar
            
    def get_Gtkbar(self):
        ## this returns a bar gtk object with top and bottom labels
        if self.progressbox is None:
            self.create_bar() 
        return self.progressbox

        
    def show(self):
        self.progressbox.show()
    
    def update_button_state(self, widget = None):
        if self.get_success() == 0:
            ## print "if self.progressbar.get_success() == 0:"
            self.progressbar.stop_it()
            self.click_counter +=1
        else:
            ## print " else: (if self.progressbar.get_success() == 0:)"
            self.click_counter += 2
            if threadmanager.threadManager.AutoDismiss == 1:
                ## print "if threadManager.AutoDismiss == 1:"
                switchboard.notify("remove progressbar",self.progressbar)
                self.schedule_gtk_removal()
            elif self.click_counter >= 2:
                ## print "if self.click_counter >= 2:"
                switchboard.notify("remove progressbar",self.progressbar)
                self.schedule_gtk_removal()
            activityViewer.sensitize()
##        switchboard.notify("sensitize activity viewer")

    def get_success(self):
        return self.progressbar.get_success()
        
    def get_target(self):
        return self.progressbar.get_message()
        

    def build_inside_text(self):
        goal = self.get_target()
        if goal is not None :
            self.gtkprogressbar.set_format_string(goal)
            self.gtkprogressbar.set_show_text(gtk.TRUE)
            
    def gtkupdate(self,value):
        self.build_inside_text()
        if self.progressbar.get_success() != 0:
            value = 1
            activityViewer.sensitize()
##            switchboard.notify("sensitize activity viewer")
        self.gtkprogressbar.update(value)
        if self.click_counter == 1 or self.progressbar.get_success() != 0 :
            self.create_dismiss_button()
        if threadmanager.threadManager.AutoDismiss == 1 \
               and self.progressbar.get_success() != 0 :
            self.update_button_state()
            
    def create_dismiss_button(self):
        self.dismissmessage = "Dismiss"
        self.update_button_label(self.dismissmessage)
        
    def updateTEXT(self, message):
        self.set_text(message)
        
    
    def update_button_label(self, text):
        if self.stopbutton.children() :
            try:
                label = self.stopbutton.children()[0]
                label.set_text(text)
            except Exception, exception:
                reporter.error(exception)


    def updateGUI(self):
       ##  print "updateGUI"
        prgrss = self.progressbar.get_progress()
        if prgrss > 1.0:                # just in case of sloppy programming
            prgrss = 1.0
        self.gtkupdate(prgrss)
        ## print self.progressbar.get_progress()
        self.destroy()
        return gtk.TRUE

    def attachGUIcallbacks(self):
        self.bar_tag = gtk.timeout_add(self.timer,self.updateGUI)


def set_ProgressBarRepresentation(pbar):
    ## argument is a ProgressBar
    ## Creates a progressbar representation, based on the mode
    ## (text or graphic) that OOF is currently running.
    if guitop.top():
        return GUIProgressBar(pbar)
    else:
        return progressbar.TextProgressBar(pbar)

progressbar.set_ProgressBarRepresentation = set_ProgressBarRepresentation
