# -*- python -*-
# $RCSfile: scrollcanvas.py,v $
# $Revision: 1.9 $
# $Author: haan $
# $Date: 2004/06/23 17:31:40 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# Wrap the OOFCanvas (defined in C++) in a scrolling window.  This
# class was created by eviscerating the RulerCanvas, which had rulers
# that scrolled along with the window.  It's quite possible that
# simply placing the OOFCanvas inside a GtkScrolledWindow would
# suffice here, and all the nonsense with the GtkTable could be
# skipped.

from SWIG.common.IO.GUI import oofcanvas
from common import debug
import gtk
import _gtk

class ScrollCanvas(oofcanvas.OOFCanvas):
    def __init__(self,
                 antialias,             # boolean
                 hScrollCB=None,        # end of scroll motion
                 vScrollCB=None):       # end of scroll motion
        debug.mainthreadTest()
        oofcanvas.OOFCanvas.__init__(self, antialias)

        self.hScrollCB = hScrollCB
        self.vScrollCB = vScrollCB

        self.hadj = gtk.GtkAdjustment(step_incr=5.0)
        self.vadj = gtk.GtkAdjustment(step_incr=5.0)
        self.set_hadjustment(self.hadj._o)
        self.set_vadjustment(self.vadj._o)
        self.scrolledwindow = gtk.GtkScrolledWindow(self.hadj, self.vadj)
        self.scrolledwindow.set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)
        self._o = self.scrolledwindow._o
        self.add_self(self._o)        

        ## Due to a bug in pygtk-1.2, a detour has been necessary.
##         hscroll = self.scrolledwindow.hscrollbar
##         vscroll = self.scrolledwindow.vscrollbar
        hscroll = gtk._obj2inst(
            _gtk.gtk_scrolled_window_get_hscrollbar(self.scrolledwindow._o))
        hscroll.connect('button-release-event', self.hScrollReleaseCB)
        vscroll = gtk._obj2inst(
            _gtk.gtk_scrolled_window_get_vscrollbar(self.scrolledwindow._o))
        vscroll.connect('button-release-event', self.vScrollReleaseCB)
        self._scrollBlock = 0  # prevents recursion in scroll callbacks

        self.scrolledwindow.show_all()

    def get_range_hadj(self):
        return self.hadj.lower, self.hadj.upper

    def get_range_vadj(self):
        return self.vadj.lower, self.vadj.upper    

    def destroy(self):
        debug.mainthreadTest()
        self.scrolledwindow.destroy()

    def set_scrollregion(self, rectangle, ppu=None):
        debug.mainthreadTest()
        if ppu is not None:
            self.set_pixels_per_unit(ppu)
        oofcanvas.OOFCanvas.set_scrollregion(self, rectangle)
        
    def set_pixels_per_unit(self, ppu):
        debug.mainthreadTest()
        oofcanvas.OOFCanvas.set_pixels_per_unit(self, ppu)

    def fullsize(self):
        debug.mainthreadTest()
        oofcanvas.OOFCanvas.fullsize(self)

    def zoom(self, factor):
        debug.mainthreadTest()
        oofcanvas.OOFCanvas.zoom(self, factor)

    def clear(self):
        debug.mainthreadTest()
        oofcanvas.OOFCanvas.clear(self)

    def size_allocateCB(self, *args):
        pass

    def _blockScrollCB(self):
        self._scrollBlock += 1

    def _unblockScrollCB(self):
        self._scrollBlock -= 1

    def _scrollBlocked(self):
        return self._scrollBlock > 0

    def hScrollReleaseCB(self, scrollbar, event):
        debug.mainthreadTest()
        if self.hScrollCB:
            self._blockScrollCB()
            self.hScrollCB(event, self.hadj.value)
            self._unblockScrollCB()

    def vScrollReleaseCB(self, scrollbar, event):
        debug.mainthreadTest()
        if self.vScrollCB:
            self._blockScrollCB()
            self.vScrollCB(event, self.vadj.value)
            self._unblockScrollCB()

    def hScrollTo(self, position):
        debug.mainthreadTest()
        if not self._scrollBlocked():
            self.hadj.set_value(position)

    def vScrollTo(self, position):
        debug.mainthreadTest()
        if not self._scrollBlocked():
            self.vadj.set_value(position)

    def hScrollPosition(self):
        return self.hadj.value

    def vScrollPosition(self):
        return self.vadj.value

    def visible_size(self):
        debug.mainthreadTest()
        return self.canvasframe.get_allocation()[2:4]
