# -*- python -*-
# $RCSfile: bitoverlaydisplay.py,v $
# $Revision: 1.19 $
# $Author: reida $
# $Date: 2004/03/30 20:46:48 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.


from SWIG.common import coord
from common import color
from common import debug
from common import registeredclass
from common.IO import bitmapdisplay
from common.IO import display
from common.IO import parameter

# The BitmapOverlayDisplayMethod is used to display objects that have
# a getBitmap() method in their Who class.  getBitmap() can return
# None, in which case nothing is drawn.  The bitmap marks pixels to be
# drawn as an overlay on the topmost image in the display.  The pixels
# are drawn with a single color and transparency.

# List of DisplayMethod classes that can have
# BitmapOverlayDisplayMethods superimposed on them.  New overlayable
# DisplayMethods should make sure to call registerOverlayableDisplay().
overlayables = [bitmapdisplay.BitmapDisplayMethod]

def registerOverlayableDisplay(displaymethod):
    overlayables.append(displaymethod)

#####

class BitmapOverlayDisplayMethod(display.DisplayMethod):
    def __init__(self, color, opacity):
        self.color = color
        self.opacity = opacity
        display.DisplayMethod.__init__(self)
    def baseObject(self, gfxwindow):
        global overlayables
        # find the object that the overlay is overlaying.
        topdisplay = gfxwindow.topMethod(*overlayables)
        if topdisplay:
            return topdisplay.getObject()
    def draw(self, gfxwindow, device):
        bitmap = self.who().resolve(gfxwindow).getBitmap()
        if bitmap is None or bitmap.empty():
            return
        bitmap.setColor(self.color)
        topbitmap = self.baseObject(gfxwindow)
        if (self.opacity == 1.0) or (topbitmap is None):
            device.draw_shaped_image(bitmap, coord.Coord(0,0),
                                     bitmap.size(), bitmap.getBG())
        else:
            ## Blend the overlay with base object.  This imitates a
            ## transparent overlay.
            blendedimage = topbitmap.makeShapedImage(bitmap, self.opacity)
            device.draw_shaped_image(blendedimage, coord.Coord(0,0),
                                     blendedimage.size(), blendedimage.getBG())

    def getTimeStamp(self, gfxwindow):
        topbitmap = self.baseObject(gfxwindow)
        if topbitmap is None:
            return self.timestamp
        else:
            return max(self.timestamp, topbitmap.getTimeStamp(gfxwindow))



bitmapOverlay = registeredclass.Registration(
    'BitmapOverlay',
    display.DisplayMethod,
    BitmapOverlayDisplayMethod,
    params=[color.ColorParameter('color'),
            parameter.FloatRangeParameter('opacity', (0., 1., 0.01), 0.6)],
    ordering=0,
    layerordering=display.SemiPlanar,
    whoclasses=('Pixel Selection', 'Active Area'))

