# -*- python -*-
# $RCSfile: array2d.py,v $
# $Revision: 1.9 $
# $Author: haan $
# $Date: 2003/04/24 17:50:58 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import copy
from common import debug
from common import primitives
from common import utils

class Array2D:
    def __init__(self, size, creator=None):
        # If creator is callable, it will be called at each array
        # element to initialize the array.  If it's not callable, a
        # copy is placed in each array element.
        self.size = size
        n = size[0] * size[1]
        # creating a flat list is much faster than creating a list of lists
        if creator:
            if callable(creator):
                self.data = [creator() for i in range(n)]
            else:
                self.data = [copy.copy(creator) for i in range(n)]
        else:
            self.data = [None]*n
            
    def __setitem__(self, where, val):
        self.data[self.size[0]*where.y + where.x] = val

    def __getitem__(self, where):
        return self.data[self.size[0]*where.y + where.x]
        
    def iterator(self):
        return self.data

    def coords(self):
        return [primitives.iPoint(i,j)
                for j in range(self.size[1])
                for i in range(self.size[0])]

    def coordsFlipped(self):
        return [primitives.iPoint(i,j)
                for j in range(self.size[1]-1, -1, -1)
                for i in range(self.size[0])]

    def reset(self,value):
        self.data = [value]*(self.size[0]*self.size[1])
