# -*- python -*-
# $RCSfile: cijklparamwidgets.py,v $
# $Revision: 1.12 $
# $Author: langer $
# $Date: 2004/07/30 19:14:21 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common.IO.GUI.matrixparamwidgets import SymmetricMatrixInput
from common import debug
from common.IO.GUI import regclassfactory
from common.IO import parameter

from engine.IO import isocijkl
from engine.IO import anisocijkl

CubicModulusCij = anisocijkl.CubicModulusCij
HexagonalModulusCij = anisocijkl.HexagonalModulusCij
TetragonalModulusCij = anisocijkl.TetragonalModulusCij
TrigonalAModulusCij = anisocijkl.TrigonalAModulusCij
TrigonalBModulusCij = anisocijkl.TrigonalBModulusCij
OrthorhombicModulusCij = anisocijkl.OrthorhombicModulusCij
MonoclinicModulusCij = anisocijkl.MonoclinicModulusCij
TriclinicModulusCij = anisocijkl.TriclinicModulusCij

from common import utils
# The CijIsoCijklWidget is quite tightly tied to the corresponding
# CijIsoCijkl class object, with which it communicates via the
# mechanism largely described in common.IO.GUI.parameterwidgets.

# The widget will be init'd with the parameters, and the value
# in base form.  It should provide a get_values() that returns
# 
class CijIsoCijklWidget(SymmetricMatrixInput):
    def __init__(self,params,base,scope=None):
        debug.mainthreadTest()
        self.params = params
        SymmetricMatrixInput.__init__(self,'C',6,6,value=None,scope=scope)
        # Block the appropriate ones, and hook up callbacks
        # to handle the c11/c12/c44 synchronization.
        for (k,f) in self.floats.items():
            if k==(0,0) or k==(0,1) or k==(3,3):
                pass
            else:
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
                
        # Callbacks to cross-connect things so that c44 can be
        # entered, and gets maintained correctly.
        self.floats[(0,0)].gtk.connect("activate",self.new_c11_or_c12,None)
        self.floats[(0,0)].gtk.connect("focus_out_event",self.new_c11_or_c12)
        #
        self.floats[(0,1)].gtk.connect("activate",self.new_c11_or_c12,None)
        self.floats[(0,1)].gtk.connect("focus_out_event",self.new_c11_or_c12)
        #
        self.floats[(3,3)].gtk.connect("activate",self.new_c44,None)
        self.floats[(3,3)].gtk.connect("focus_out_event",self.new_c44)    

        self.set_values()

    # Main setting routine.  Use this to guarantee consistency
    # at all times.  Takes c11 and c12, assigns the "values" string
    # (possibly redundantly), computes c44, and writes the duplicate
    # values into the inactive fields of the widget.
    def draw_values(self,c11,c12):
        debug.mainthreadTest()
        self.values = [c11, c12]
        c44 = 0.5*(c11-c12)
        #
        self.floats[(0,0)].set_value(c11)
        self.floats[(1,1)].set_value(c11)
        self.floats[(2,2)].set_value(c11)
        #
        self.floats[(0,1)].set_value(c12)
        self.floats[(0,2)].set_value(c12)
        self.floats[(1,2)].set_value(c12)
        #
        self.floats[(3,3)].set_value(c44)
        self.floats[(4,4)].set_value(c44)
        self.floats[(5,5)].set_value(c44)
        
    # This widget understands its "values" to be c11 and c12,
    # in that order, for both setting and getting.
    def set_values(self, values=None):
        self.values = values or [p.value for p in self.params]
        self.draw_values(self.values[0], self.values[1])
        
    def get_values(self):
        for p, v in map(None, self.params, self.values):
            p.value = v

    # Callbacks.  Called on return or focus_out.
    def new_c11_or_c12(self,gtk,event):
        c11 = self.floats[(0,0)].get_value()
        c12 = self.floats[(0,1)].get_value()
        self.draw_values(c11,c12)

    def new_c44(self,gtk,event):
        c11 = self.floats[(0,0)].get_value()
        c44 = self.floats[(3,3)].get_value()
        c12 = c11-2.0*c44
        self.draw_values(c11,c12)

    def destroy(self):
        debug.mainthreadTest()
        self.gtk.destroy()

regclassfactory.addWidget(isocijkl.IsoModulusParameter,
                          isocijkl.IsoModulusCij, CijIsoCijklWidget)

############################################################################
############################################################################

class CijCubicCijklWidget(SymmetricMatrixInput):
    def __init__(self,params,base,scope=None):
        debug.mainthreadTest()
        self.params = params
        SymmetricMatrixInput.__init__(self,'C',6,6,value=None,scope=scope)
        # Block the appropriate ones, and hook up callbacks
        # to handle the c11/c12/c44 synchronization.
        for (k,f) in self.floats.items():
            if k==(0,0) or k==(0,1) or k==(3,3):
                pass
            else:
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
                
        # Callbacks to cross-connect things so that c44 can be
        # entered, and gets maintained correctly.
        self.floats[(0,0)].gtk.connect("activate", self.new_c, None)
        self.floats[(0,0)].gtk.connect("focus_out_event", self.new_c)
        #
        self.floats[(0,1)].gtk.connect("activate", self.new_c, None)
        self.floats[(0,1)].gtk.connect("focus_out_event", self.new_c)
        #
        self.floats[(3,3)].gtk.connect("activate", self.new_c, None)
        self.floats[(3,3)].gtk.connect("focus_out_event", self.new_c)
        #
        self.set_values()

    # Main setting routine.  Use this to guarantee consistency
    # at all times.  Takes c11 and c12, assigns the "values" string
    # (possibly redundantly), computes c44, and writes the duplicate
    # values into the inactive fields of the widget.
    def draw_values(self,c11,c12,c44):
        debug.mainthreadTest()
        self.values = [c11, c12, c44]
        #
        self.floats[(0,0)].set_value(c11)
        self.floats[(1,1)].set_value(c11)
        self.floats[(2,2)].set_value(c11)
        #
        self.floats[(0,1)].set_value(c12)
        self.floats[(0,2)].set_value(c12)
        self.floats[(1,2)].set_value(c12)
        #
        self.floats[(3,3)].set_value(c44)
        self.floats[(4,4)].set_value(c44)
        self.floats[(5,5)].set_value(c44)
        
    # This widget understands its "values" to be c11, c12, and c44,
    # in that order, for both setting and getting.
    def set_values(self, values=None):
        self.values = values or [p.value for p in self.params]
        self.draw_values(self.values[0], self.values[1], self.values[2])
        
    def get_values(self):
        for p, v in map(None, self.params, self.values):
            p.value = v

    # Callbacks.  Called on return or focus_out.
    def new_c(self,gtk,event):
        c11 = self.floats[(0,0)].get_value()
        c12 = self.floats[(0,1)].get_value()
        c44 = self.floats[(3,3)].get_value()
        self.draw_values(c11,c12,c44)

    def destroy(self):
        debug.mainthreadTest()
        self.gtk.destroy()

regclassfactory.addWidget(anisocijkl.CubicModulusParameter,
                          anisocijkl.CubicModulusCij,
                          CijCubicCijklWidget)


###########################################################################
###########################################################################
#
# Base class for lower-symmetry Cijkl widgets.  These have
# special behavior, because they're not convertible registered
# types.  The base class also serves as an enforcement mechanism
# for the dictionary-of-values scheme for passing things around.
#
# Child classes must pass in a dictionary ('kset') 
# tuples corresponding to inputs, and value'd by strings corresponding
# to the attribute names in the corresponding Cijkl value class.
class AnisoWidgetBase(SymmetricMatrixInput):
    def __init__(self,params,kset,scope=None):
        debug.mainthreadTest()
        self.params = params
        self.kset = kset
        SymmetricMatrixInput.__init__(self,'C',6,6,value=None,scope=scope)
        #
        # Make default blocks according to the kset dictionary.
        for (k,f) in self.floats.items():
            if self.kset.has_key(k):
                pass
            else:
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
        #
        # Connect the "obvious" callbacks.  Child classes may add
        # additional callbacks.
        for k in self.kset.keys():
            # "activate" needs a dummy argument for the "event" slot.
            self.floats[k].gtk.connect("activate",self.new_c,None)
            self.floats[k].gtk.connect("focus_out_event",self.new_c)

    # If no default value is set in the registration entry for the
    # property which has this as a value, then this will get called
    # with "None" as the argument, and with no parameters set.
    # "set_values" should probably throw an exception when this
    # happens.
    def set_values(self, values=None):
        self.value = values or self.params.value
        v_dict = {}
        for (k,v) in self.kset.items():
            v_dict[v] = self.value.__dict__[v]
        self.draw_values(v_dict)

    # Unlike the convertible case, this routine must return the value.
    def get_value(self):
        self.params.value = self.value
        return self.value

    # Generic callback can live in the base class.
    def new_c(self,gtk,event):
        v_dict = {}
        for (k,v) in self.kset.items():
            v_dict[v] = self.floats[k].get_value()
        self.draw_values(v_dict)

    def destroy(self):
        debug.mainthreadTest()
        self.gtk.destroy()
        
# # # # # # # # # ################################### # # # # # # # # # # 
# Hexagonal


class HexagonalCijklWidget(AnisoWidgetBase):
    def __init__(self,params,scope=None):
        debug.mainthreadTest()
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13',
                (2,2): 'c33', (3,3): 'c44' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        # Base class will have blocked c66, so unblock it.
        self.floats[(5,5)].gtk.set_editable(1)
        self.floats[(5,5)].gtk.set_sensitive(1)
            
        #  ... and connect a special callback.
        self.floats[(5,5)].gtk.connect("activate", self.new_c66, None)
        self.floats[(5,5)].gtk.connect("focus_out_event", self.new_c66)
        #
        self.set_values()

    # c11 updates c22, c44 updates c55, c13 updates c23, and
    # c66=0.5*(c11-c12)
    def draw_values(self,var_dict):
        debug.mainthreadTest()
        self.value = HexagonalModulusCij(var_dict['c11'], var_dict['c12'],
                                         var_dict['c13'], var_dict['c33'],
                                         var_dict['c44'])
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

        # Then do the extra, special ones.
        self.floats[(1,1)].set_value(var_dict['c11'])
        self.floats[(1,2)].set_value(var_dict['c13'])
        self.floats[(4,4)].set_value(var_dict['c44'])
        self.floats[(5,5)].set_value(0.5*(var_dict['c11']-var_dict['c12']))
        
    def new_c66(self,gtk,event):
        v_dict = {}
        for (k,v) in self.kset.items():
            v_dict[v] = self.floats[k].get_value()
        c11 = v_dict['c11']
        c66 = self.floats[(5,5)].get_value()
        v_dict['c12'] = c11-2.0*c66
        self.draw_values(v_dict)


def HexCijklParam_makeWidget(self, scope):
    return HexagonalCijklWidget(self, scope)

anisocijkl.HexagonalCijklParameter.makeWidget = HexCijklParam_makeWidget
    

# # # # # # # # # ################################### # # # # # # # # # # 
# Tetragonal.

class TetragonalCijklWidget(AnisoWidgetBase):
    def __init__(self, params, scope=None):
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13', (0,5): 'c16',
                (2,2): 'c33', (3,3): 'c44', (5,5): 'c66' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        self.set_values()

    # Tetragonal case, c13 updates c23, and c11 updates c12, and
    # c44 updates c55.
    # "var_dict" is indexed by the strings which are values in
    # self.kset.
    def draw_values(self,var_dict):
        debug.mainthreadTest()
        self.value = TetragonalModulusCij(var_dict['c11'], var_dict['c12'],
                                          var_dict['c13'], var_dict['c33'],
                                          var_dict['c44'], var_dict['c66'],
                                          var_dict['c16'])
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

        # Then do the extra, special ones.
        self.floats[(1,1)].set_value(var_dict['c11'])
        self.floats[(1,2)].set_value(var_dict['c13'])
        self.floats[(4,4)].set_value(var_dict['c44'])
        self.floats[(1,5)].set_value(var_dict['c16'])
        
def TetCijklParam_makeWidget(self, scope):
    return TetragonalCijklWidget(self, scope)

anisocijkl.TetragonalCijklParameter.makeWidget = TetCijklParam_makeWidget

# # # # # # # # # ################################### # # # # # # # # # #
# Trigonal A

class TrigonalACijklWidget(AnisoWidgetBase):
    def __init__(self, params, scope=None):
        debug.mainthreadTest()
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13',
                (2,2): 'c33', (3,3): 'c44', (0,3): 'c14', (0,4): 'c15' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        self.floats[(5,5)].gtk.set_editable(1)
        self.floats[(5,5)].gtk.set_sensitive(1)
        #
        self.floats[(5,5)].gtk.connect("activate", self.new_c66, None)
        self.floats[(5,5)].gtk.connect("focus_out_event", self.new_c66)
        #
        self.set_values()

    # TrigonalA.  c11 updates c22, c13 updates c23, c44 updates c55, and
    # c66 = 0.5*(c11-c12).  
    def draw_values(self,var_dict):
        debug.mainthreadTest()
        self.value = TrigonalAModulusCij(var_dict['c11'], var_dict['c12'],
                                         var_dict['c13'], var_dict['c33'],
                                         var_dict['c44'], var_dict['c14'],
                                         var_dict['c15'])
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

        # Then do the extra, special ones.
        self.floats[(1,1)].set_value(var_dict['c11'])
        self.floats[(1,2)].set_value(var_dict['c13'])
        self.floats[(4,4)].set_value(var_dict['c44'])
        # Propagate c14 to the subsidiary entries.
        self.floats[(1,3)].set_value(-var_dict['c14'])
        self.floats[(4,5)].set_value(var_dict['c14'])
        # And similarly, c15.
        self.floats[(1,4)].set_value(-var_dict['c15'])
        self.floats[(3,5)].set_value(-var_dict['c15'])
        #
        c66=0.5*(var_dict['c11']-var_dict['c12'])
        self.floats[(5,5)].set_value(c66)

    def new_c66(self, gtk, event):
        v_dict = {}
        for (k,v) in self.kset.items():
            v_dict[v] = self.floats[k].get_value()
        c11 = v_dict['c11']
        c66 = self.floats[(5,5)].get_value()
        v_dict['c12'] = c11-2.0*c66
        self.draw_values(v_dict)
   
def TrigACijklParam_makeWidget(self, scope):
    return TrigonalACijklWidget(self, scope)

anisocijkl.TrigonalACijklParameter.makeWidget = TrigACijklParam_makeWidget



# # # # # # # # # ################################### # # # # # # # # # #
# Trigonal B

class TrigonalBCijklWidget(AnisoWidgetBase):
    def __init__(self, params, scope=None):
        debug.mainthreadTest()
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13',
                (2,2): 'c33', (3,3): 'c44', (0,3): 'c14' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        self.floats[(5,5)].gtk.set_editable(1)
        self.floats[(5,5)].gtk.set_sensitive(1)
        #
        self.floats[(5,5)].gtk.connect("activate", self.new_c66, None)
        self.floats[(5,5)].gtk.connect("focus_out_event", self.new_c66)
        #
        self.set_values()

    # TrigonalB.  c11 updates c22, c13 updates c23, c44 updates c55, and
    # c66 = 0.5*(c11-c12).  
    def draw_values(self,var_dict):
        debug.mainthreadTest()
        self.value = TrigonalBModulusCij(var_dict['c11'], var_dict['c12'],
                                         var_dict['c13'], var_dict['c33'],
                                         var_dict['c44'], var_dict['c14'])
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

        # Then do the extra, special ones.
        self.floats[(1,1)].set_value(var_dict['c11'])
        self.floats[(1,2)].set_value(var_dict['c13'])
        self.floats[(4,4)].set_value(var_dict['c44'])
        # Propagate c14 to the subsidiary entries.
        self.floats[(1,3)].set_value(-var_dict['c14'])
        self.floats[(4,5)].set_value(var_dict['c14'])
        #
        c66=0.5*(var_dict['c11']-var_dict['c12'])
        self.floats[(5,5)].set_value(c66)

    def new_c66(self, gtk, event):
        v_dict = {}
        for (k,v) in self.kset.items():
            v_dict[v] = self.floats[k].get_value()
        c11 = v_dict['c11']
        c66 = self.floats[(5,5)].get_value()
        v_dict['c12'] = c11-2.0*c66
        self.draw_values(v_dict)
   
def TrigBCijklParam_makeWidget(self, scope):
    return TrigonalBCijklWidget(self, scope)

anisocijkl.TrigonalBCijklParameter.makeWidget = TrigBCijklParam_makeWidget



# # # # # # # # # ################################### # # # # # # # # # #
# Orthorhombic

class OrthorhombicCijklWidget(AnisoWidgetBase):
    def __init__(self, params, scope=None):
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13',
                (1,1): 'c22', (1,2): 'c23',
                (2,2): 'c33', (3,3): 'c44', (4,4): 'c55',
                (5,5): 'c66' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        self.set_values()

    # Orthorhombic.  No cross-connections, just straight entries.
    def draw_values(self,var_dict):
        self.value = OrthorhombicModulusCij(var_dict['c11'], var_dict['c12'],
                                            var_dict['c13'], var_dict['c22'],
                                            var_dict['c23'], var_dict['c33'],
                                            var_dict['c44'], var_dict['c55'],
                                            var_dict['c55'])
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

def OrthCijklParam_makeWidget(self, scope):
    return OrthorhombicCijklWidget(self, scope)

anisocijkl.OrthorhombicCijklParameter.makeWidget = OrthCijklParam_makeWidget


# # # # # # # # # ################################### # # # # # # # # # # 
# Monoclinic

class MonoclinicCijklWidget(AnisoWidgetBase):
    def __init__(self, params, scope=None):
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13', (0,4): 'c15',
                (1,1): 'c22', (1,2): 'c23', (1,4): 'c25',
                (2,2): 'c33', (2,4): 'c35', 
                (3,3): 'c44', (3,5): 'c46', (4,4): 'c55', (5,5): 'c66' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        self.set_values()

    # Monoclinic.  No cross-connections, just straight entries.
    def draw_values(self,var_dict):
        self.value = MonoclinicModulusCij(var_dict['c11'], var_dict['c12'],
                                          var_dict['c13'], var_dict['c15'],
                                          var_dict['c22'], var_dict['c23'],
                                          var_dict['c25'], var_dict['c33'],
                                          var_dict['c35'], var_dict['c44'],
                                          var_dict['c46'], var_dict['c55'],
                                          var_dict['c66'])
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

        
def MonoCijklParam_makeWidget(self, scope):
    return MonoclinicCijklWidget(self, scope)

anisocijkl.MonoclinicCijklParameter.makeWidget = MonoCijklParam_makeWidget


# # # # # # # # # ################################### # # # # # # # # # # 
# Triclinic, the general case.

class TriclinicCijklWidget(AnisoWidgetBase):
    def __init__(self, params, scope=None):
        kset = {(0,0): 'c11', (0,1): 'c12', (0,2): 'c13', (0,3): 'c14',
                (0,4): 'c15', (0,5): 'c16', (1,1): 'c22', (1,2): 'c23',
                (1,3): 'c24', (1,4): 'c25', (1,5): 'c26', (2,2): 'c33',
                (2,3): 'c34', (2,4): 'c35', (2,5): 'c36', (3,3): 'c44',
                (3,4): 'c45', (3,5): 'c46', (4,4): 'c55', (4,5): 'c56',
                (5,5): 'c66' }
        AnisoWidgetBase.__init__(self, params, kset, scope=scope)
        #
        self.set_values()

    # Monoclinic.  No cross-connections, just straight entries.
    def draw_values(self,var_dict):
        self.value = TriclinicModulusCij(var_dict['c11'], var_dict['c12'],
                                         var_dict['c13'], var_dict['c14'],
                                         var_dict['c15'], var_dict['c16'],
                                         var_dict['c22'],
                                         var_dict['c23'], var_dict['c24'],
                                         var_dict['c25'], var_dict['c26'],
                                         var_dict['c33'], var_dict['c34'],
                                         var_dict['c35'], var_dict['c36'],
                                         var_dict['c44'], var_dict['c45'],
                                         var_dict['c46'],
                                         var_dict['c55'], var_dict['c56'],
                                         var_dict['c66'],
                                    )
        #
        for (k,v) in self.kset.items():
            self.floats[k].set_value(var_dict[v])

        
def TriCijklParam_makeWidget(self, scope):
    return TriclinicCijklWidget(self, scope)

anisocijkl.TriclinicCijklParameter.makeWidget = TriCijklParam_makeWidget

