# -*- python -*-
# $RCSfile: meshcstoolboxGUI.py,v $
# $Revision: 1.22 $
# $Author: langer $
# $Date: 2004/09/21 19:23:57 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# GUI part of the Mesh cross-section toolbox.  Collects mouse
# clicks, draws paths, etc.

from SWIG.common import switchboard
from SWIG.common.IO.GUI import rubberband
from common import debug
from common import guitop
from common import primitives
from common import utils
from common.IO.GUI import chooser
from common.IO.GUI import destinationparamwidget
from common.IO.GUI import parameterwidgets
from common.IO.GUI import mousehandler
from common.IO.GUI import toolboxGUI
from engine import mesh
from engine.IO import analyzemenu
from engine.IO import meshcstoolbox
from engine.IO import meshmenu
import gtk

## TODO2.0: Get rid of the Autodisplay button.  Add "<None>" to the
## list of cross sections, so that layers that are displaying the
## Selected cross section can be made to disappear.

class CrossSectionToolboxGUI(toolboxGUI.GfxToolbox,
                             mousehandler.MouseHandler):
    def __init__(self, toolbox):
        toolboxGUI.GfxToolbox.__init__(
            self, utils.underscore2space(toolbox.name()), toolbox)
        self.gtk = gtk.GtkFrame()

        # The menu item called by the "Go" button.
        self.menuitem = analyzemenu.ops_menu.Cross_Section

        mainbox = gtk.GtkVBox()

        sourceframe = gtk.GtkFrame("Source")
        mainbox.add(sourceframe, fill=0, expand=0)
        sourcescroll = gtk.GtkScrolledWindow()
        sourceframe.add(sourcescroll)
        sourcescroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
        datatable = gtk.GtkTable(rows=2,cols=2)
        sourcescroll.add_with_viewport(datatable)
        
        meshlabel = gtk.GtkLabel("mesh = ")
        meshlabel.set_alignment(1.0, 0.5)
        self.meshname = gtk.GtkLabel()
        self.meshname.set_alignment(0.0, 0.5)

        datatable.attach(meshlabel, 0, 1, 0, 1)
        datatable.attach(self.meshname, 1, 2, 0, 1)

        layerlabel = gtk.GtkLabel("output = ")
        layerlabel.set_alignment(1.0, 0.5)
        self.layername = gtk.GtkLabel()
        self.layername.set_alignment(0.0, 0.5)

        datatable.attach(layerlabel, 0, 1, 1, 2)
        datatable.attach(self.layername, 1, 2, 1, 2)


        csframe = gtk.GtkFrame("Cross Section")
        mainbox.add(csframe, expand=1, fill=1)
        csbox = gtk.GtkVBox()
        csframe.add(csbox)

        # Widget which shows the name of the current cross-section.
        hbox = gtk.GtkHBox()
        csbox.add(hbox, expand=0, fill=0)
        hbox.add(gtk.GtkLabel("current: "), expand=0, fill=0)
        self.csChooser = chooser.ChooserWidget([], callback=self.csChooserCB)
        hbox.add(self.csChooser.gtk, expand=1, fill=1, padding=2)


        # Widget for number of points.
        pointbox = gtk.GtkHBox()
        pointbox.add(gtk.GtkLabel("points: "), expand=0, fill=0)
        self.npointswidget = self.menuitem.get_arg('points').makeWidget()
        self.int_valid_swcb = switchboard.requestCallback(
            ('validity', self.npointswidget), self.validCB)
        pointbox.add(self.npointswidget.gtk)
        csbox.add(pointbox,fill=0,expand=0)

        hbox = gtk.GtkHBox()
        csbox.add(hbox, expand=0, fill=0, padding=2)
        # Rename button.
        self.renamebutton = gtk.GtkButton("Rename")
        self.renamebutton.connect('clicked', self.csrenameCB)
        guitop.top().tooltip(self.renamebutton, "Rename the current cross-section.")
        hbox.add(self.renamebutton,fill=1,expand=1, padding=2)
        # Edit button
        self.editbutton = gtk.GtkButton("Edit...")
        self.editbutton.connect('clicked', self.cseditCB)
        guitop.top().tooltip(self.editbutton, "Edit the current cross-section.")
        hbox.add(self.editbutton, fill=1, expand=1, padding=2)
        # Delete button.
        self.csdeletebutton = gtk.GtkButton("Remove")
        self.csdeletebutton.connect("clicked", self.csdeleteCB)
        guitop.top().tooltip(self.csdeletebutton, "Remove the current cross-section.")
        hbox.add(self.csdeletebutton,fill=1,expand=1, padding=2)

        self.autodisplayButton = gtk.GtkCheckButton('Autodisplay')
        guitop.top().tooltip(self.autodisplayButton,
                             "Automatically display the selected cross section. Or not.")
        csbox.add(self.autodisplayButton, expand=0, fill=0)
        self.autodisplayButton.set_active(self.toolbox.autodisplay)
        self.autodisplayButton.connect('clicked', self.autodisplayCB)

        goframe = gtk.GtkFrame("Output")
        mainbox.add(goframe,expand=0,fill=0)
        self.gobox = gtk.GtkVBox()
        goframe.add(self.gobox)

        hbox = gtk.GtkHBox()
        self.gobox.add(hbox, expand=0, fill=0)
        label = gtk.GtkLabel("Destination: ")
        label.set_alignment(1.0, 0.5)
        hbox.add(label, expand=0, fill=0)
        self.destwidget = self.menuitem.get_arg("destination").makeWidget()
        self.dw_valid_swcb = switchboard.requestCallback(
            ('validity', self.destwidget), self.validCB )
        hbox.add(self.destwidget.gtk, expand=1, fill=1, padding=2)
        self.gobutton = gtk.GtkButton("Go!")
        hbox.add(self.gobutton,expand=1,fill=1, padding=2)
        guitop.top().tooltip(self.gobutton, "Send the output to the destination.")
        self.gobutton.connect("clicked", self.goCB)
        
        self.gtk.add(mainbox)
        self.startpoint = None
        self.endpoint = None

        # Data needed by the "go" button.  Set in "show_data", when
        # the state of all the widgets is examined.
        self.meshobj = None
        self.current_cs_name = None
        
        # Shut off non-GUI toolbox's switchboard callbacks.  We will
        # take them over at activate-time.
        self.toolbox.stop_callbacks()

    def __del__(self):
        switchboard.removeCallback(self.dw_valid_swcb)
        self.dw_valid_swcb = None
            
    def close(self):
        toolboxGUI.GfxToolbox.close(self)

    def activate(self):
        self.toolbox.activate()
        self.gfxwindow().setMouseHandler(self)
        self.gfxwindow().setRubberband(rubberband.LineRubberBand())
        self.sb_callbacks = [
            switchboard.requestCallback( (self.gfxwindow(),
                                          "layers changed"),
                                         self.newLayers ),
            switchboard.requestCallback( (self.gfxwindow(),
                                          "new contourmap layer"),
                                         self.newLayers),
            # Need this signal to update the "current cross section" datum.
            switchboard.requestCallback( "cross sections changed",
                                         self.show_data)
            ]
        self.show_data()

    def deactivate(self):
        self.gfxwindow().removeMouseHandler()
        self.gfxwindow().setRubberband(rubberband.NoRubberBand())
        for s in self.sb_callbacks:
            switchboard.removeCallback(s)
        self.sb_callbacks = []
        
    def validCB(self, valid):
        self.show_data()

    def csChooserCB(self, gtkmenuitem, csname):
        self.toolbox.selectCS(csname)
        
    def show_data(self):
        self.meshobj = self.toolbox.current_mesh
        meshok = self.meshobj is not None
        if meshok:
            text = mesh.meshes.getPath(self.meshobj)
        else:
            text = ""
        self.meshname.set_text(text)

        csok = 0
        if meshok:
            csname = self.meshobj.selectedCSName()
            csnames = self.meshobj.allCrossSectionNames()
            self.csChooser.update(csnames)
            if csname:
                self.csChooser.set_state(csname)
            self.current_cs_name = self.csChooser.get_value()
            csok = self.current_cs_name is not None
        self.csdeletebutton.set_sensitive(csok)
        self.renamebutton.set_sensitive(csok)
        self.editbutton.set_sensitive(csok)
                
        self.outputobj = self.toolbox.current_layer
        if self.outputobj is None:
            text =""
        else:
            text = self.outputobj.what.short_repr()
        self.layername.set_text(text)

        self.gobutton.set_sensitive(self.outputobj is not None
                                    and meshok and csok
                                    and self.destwidget.isValid())
    def newLayers(self):
        self.toolbox.newLayers()
        self.show_data()

    # Button callback for cross-section deletion -- removes the
    # current cross-section from the current mesh.  
    def csdeleteCB(self, gtkobj):
        meshobj = self.toolbox.current_mesh
        csname = meshobj.selectedCSName()
        menuitem = meshmenu.csmenu.Remove
        menuitem.callWithDefaults(
            mesh=meshobj.path(), name=csname)

    # Callback for the "Rename" button
    def csrenameCB(self, gtkobj):
        meshobj = self.toolbox.current_mesh
        csname = meshobj.selectedCSName()
        menuitem = meshmenu.csmenu.Rename
        newnameparam = menuitem.get_arg('name')
        if parameterwidgets.getParameterValues(
            newnameparam, title="Rename cross section " + csname):
            menuitem.callWithDefaults(mesh=meshobj.path(), cross_section=csname)

    def cseditCB(self, gtkobj):
        meshobj = self.toolbox.current_mesh
        csname = meshobj.selectedCSName()
        menuitem = meshmenu.csmenu.Edit
        csparam = menuitem.get_arg('cross_section')
        if parameterwidgets.getParameterValues(
            csparam,
            title='Edit cross section ' + csname):
            menuitem.callWithDefaults(mesh=meshobj.path(), name=csname)
            
    def autodisplayCB(self, button):
        self.toolbox.menu.AutoDisplay(self.toolbox.autodisplay ^ 1)

    # Actually write an output to the destination.
    def goCB(self, gtkobj):
        meshname = mesh.meshes.getPath(self.meshobj)
        output = self.toolbox.current_layer.what
        cs = self.current_cs_name
        self.menuitem.get_arg(
            'points').value = self.npointswidget.get_value()
        self.menuitem.get_arg(
            'destination').value = self.destwidget.get_value()
        self.menuitem.callWithDefaults(
            mesh=meshname, output=output, cross_section=cs)
        
    # Mouse-handler stuff.
    def acceptEvent(self, event):
        return event=='up' or event=='down'
    def down(self,x,y,shift,ctrl):
        self.startpoint = primitives.Point(x,y)
    def up(self,x,y,shift,ctrl):
        self.endpoint = primitives.Point(x,y)
        self.toolbox.makeCS(self.startpoint, self.endpoint)





def _makeGUI(self):
    return CrossSectionToolboxGUI(self)

meshcstoolbox.CrossSectionToolbox.makeGUI = _makeGUI
