# -*- python -*-
# $RCSfile: meshcsdisplay.py,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2004/09/16 19:14:18 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import ooferror
from common import color
from common import primitives
from common import registeredclass
from common.IO import automatic
from common.IO import display
from common.IO import parameter
from common.IO import selectionplaceholder
from engine.IO import meshcsparams
import types

# Layer for showing the mesh cross-sections.
class MeshCrossSectionDisplay(display.DisplayMethod):
    def __init__(self, cross_sections, color, linewidth):
        display.DisplayMethod.__init__(self)
        self.cross_sections = cross_sections
        self.color = color
        self.linewidth = linewidth
        
    # We might need to be redrawn when the mesh's cross-sections have
    # changed.
    def getTimeStamp(self, gfxwindow):
        mesh = self.who().resolve(gfxwindow)
        return max(self.timestamp, mesh.cross_sections.timestamp,
                   gfxwindow.getToolboxByName('Mesh_Cross_Section').autodisptime
                   )


    def draw(self, gfxwindow, device):
        mesh = self.who().resolve(gfxwindow)
        device.set_lineColor(self.color)
        device.set_lineWidth(self.linewidth)

        if self.cross_sections==selectionplaceholder.selectionPlaceHolder:
            cstoolbox = gfxwindow.getToolboxByName('Mesh_Cross_Section')
            if cstoolbox.autodisplay:
                cs = mesh.selectedCS()
                if cs:
                    device.draw_segment(
                        primitives.Segment(cs.start, cs.end))
                
        else: # List of cs names.
            for k in self.cross_sections:
                try:
                    b = mesh.cross_sections[k]
                except KeyError:
                    pass
                else:
                    device.draw_segment(
                        primitives.Segment(b.start, b.end) )
    def __hash__(self): # Override hash, because we contain lists
        return id(self)
        

                  
meshCrossSectionDisplay = registeredclass.Registration(
    'Cross Section', display.DisplayMethod,
    MeshCrossSectionDisplay,
    params=[
    meshcsparams.MeshCrossSectionSetParameter(
    'cross_sections', selectionplaceholder.selectionPlaceHolder),
    color.ColorParameter('color', value=color.gray50),
    parameter.IntRangeParameter('linewidth', (0, 10), 1)],
    ordering=5.0,
    layerordering=display.SemiLinear,
    whoclasses=('Mesh',),
    tip="Determine which cross sections are displayed, and how.")

display.predefineLayer('Mesh', '<contourable>',
                       meshCrossSectionDisplay())
