# -*- python -*-
# $RCSfile: propertymenu.py,v $
# $Revision: 1.13 $
# $Author: langer $
# $Date: 2004/09/22 20:46:00 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# Menu for operations related to the PropertyTree.  Provides
# for the selection and deselection of properties in the tree,
# creation and deletion of instances, incorporation into
# properties materials, and parametrization.  Saving and loading
# are handled by the main OOF "File" submenu.

# These menu operations need not correspond to entries
# in a menu bar -- they're triggered from the GUI by operations
# on the non-menubar buttons of the MaterialsPane.

from SWIG.common.ooferror import ErrUserError
from common import enum
from common.IO import automatic
from common.IO import datafile
from common.IO import parameter
from common.IO.oofmenu import OOFMenuItem

Parameter = parameter.Parameter
StringParameter = parameter.StringParameter

# This file adds ".Property" to "OOF".
from engine import propertyregistration
AllProperties = propertyregistration.AllProperties


from common.IO.mainmenu import OOF # The root of all menus.

def propertyNameResolver(param, startname):
    if param.automatic():
        basename='instance'
    else:
        basename=startname
    contextname = param.group['property'].value
    return AllProperties.uniqueName(contextname, basename)


def _copywrapper(menuitem, property, new_name):
    try:
        AllProperties.new_prop(property, new_name)
    except ErrUserError, e:
        print e

OOF.Property.addItem(OOFMenuItem('Copy',
                                 callback=_copywrapper,
                                 params=parameter.ParameterGroup(
    StringParameter('property'),
    parameter.AutomaticNameParameter('new_name',
                                     resolver=propertyNameResolver,
                                     value=automatic.automatic) ),
                                 help="Copy a Property."))


def _deletewrapper(menuitem, property):
    try:
        AllProperties.delete(property)
    except ErrUserError, e:
        print e


OOF.Property.addItem(OOFMenuItem('Delete',
                                 callback=_deletewrapper,
                                 params=[StringParameter('property')],
                                 help="Delete a Property."))


####################################

# Save Properties to a data file.  The corresponding menu items for
# reading them are constructed by the PropertyManager.

def saveProperty(menuitem, filename, mode, format, property):
    dfile = datafile.writeDataFile(filename, mode, format)
    propertyreg = AllProperties[property]
    propertyreg.writeData(dfile)
    dfile.close()

OOF.File.Save.addItem(OOFMenuItem(
    'Property',
    callback=saveProperty,
    gui_only=1,
    params=[
    parameter.StringParameter('filename'),
    parameter.StringParameter('mode'),
    enum.EnumParameter('format', datafile.DataFileFormat, datafile.ASCII),
    parameter.StringParameter('property')
    ],
    help="Save the indicated Property to a file."))
