# -*- python -*-
# $RCSfile: invariant_.py,v $
# $Revision: 1.3 $
# $Author: langer $
# $Date: 2004/06/14 20:10:51 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.engine import outputval
from SWIG.engine import symmmatrix
from common import cregisteredclass
from common import debug
from common.IO import parameter

cregisteredclass.registerCClass(InvariantPtr)

# Maintain lists of which Invariants can be applied to which kinds of
# OutputVals.  The keys to the _invmap dict are OutputValPtr classes.
# The values are lists of Invariant Registrations.

_invmap = {}                           
def _newInvariant():
    global _invmap
    _invmap = {}
    for registration in InvariantPtr.registry:
        for outvalclass in registration.sources:
            try:
                invlist = _invmap[outvalclass]
            except KeyError:
                invlist = []
                _invmap[outvalclass] = invlist
            invlist.append(registration)

switchboard.requestCallback(InvariantPtr, _newInvariant)

def okInvariant(registration, outvalclass):
    try:
        return registration in _invmap[outvalclass]
    except KeyError:
        return 0
    
##########################

cregisteredclass.Registration(
    "Magnitude",
    InvariantPtr,
    Magnitude,
    ordering=0,
    sources=[outputval.VectorOutputValPtr])

cregisteredclass.Registration(
    "Trace",
    InvariantPtr,
    MatrixTrace,
    ordering=1,
    sources=[symmmatrix.SymmMatrix3Ptr])

cregisteredclass.Registration(
    "SecondInvariant",
    InvariantPtr,
    SecondInvariant,
    ordering=2,
    sources=[symmmatrix.SymmMatrix3Ptr])

cregisteredclass.Registration(
    "Determinant",
    InvariantPtr,
    Determinant,
    ordering=2,
    sources=[symmmatrix.SymmMatrix3Ptr])

###############################

class InvariantParameter(parameter.RegisteredParameter):
    def __init__(self, name, value=None, default=None, tip=None):
        parameter.RegisteredParameter.__init__(self, name, InvariantPtr,
                                               value, default, tip)
    def clone(self):
        return self.__class__(self.name, self.value, self.default, self.tip)
