// -*- C++ -*-
// $RCSfile: meshqualityexpansion.h,v $
// $Revision: 1.2 $
// $Author: redwing $
// $Date: 2004/09/20 19:40:17 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef QUALITYEXPANSION_H
#define QUALITYEXPANSION_H

// Base class for Thermal Expansion Properties

#include "engine/matvec.h"
#include "engine/property.h"
#include "engine/symmmatrix.h"
#include <string>

class Elasticity;
class Element;
class Flux;
class FluxData;
class Material;
class Position;
class OrientationProp;
class OutputVal;
class PropertyOutput;
class ScalarField;
class SymmetricTensorFlux;

// Property analogous to the thermal expansion tensor.
// It's purpose is to set a expand/contract the skeleton
// element node positions, based on the quality of the mesh
// energy shape/homogeneity.

class MeshExpansion : public Property {
private:
  // Physical parameters.
  SymmMatrix3 alpha_; 
  SymmMatrix3 lab_alpha_;
  double tzero_;
  Elasticity *elasticity;	// pointer to this Material's Elasticity
  ScalarField *temperature;
  SymmetricTensorFlux *stress_flux;
  const OrientationProp *orientation;
public:
  MeshExpansion(PyObject *registry, const std::string &nm, 
		       SymmMatrix3 *alpha, double t0);
  virtual ~MeshExpansion() {}
  virtual void cross_reference(Material*);
  virtual void precompute();

  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*, const MasterPosition&) const;
  virtual void fluxrhs(const FEMesh*, const Element*,
		       const Flux*, FluxData*, const MasterPosition&) const;
  virtual void output(const FEMesh*, const Element*, const PropertyOutput*,
		      const MasterPosition&, OutputVal*) const;

  virtual int K_integration_order(int) const;

  const SymmMatrix3 *alpha() const { return &alpha_; }
  double tzero() const { return tzero_; }
  // coefficients of thermal expansion to be defined in subclasses
  // virtual const MATRIX_D &cte() const = 0;
};

#endif
