# This file was created automatically by SWIG.
import solverstepperc
class SolverStepperCorePtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def precompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_precompute,(self,) + _args, _kwargs)
        return val
    def solve(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_solve,(self,) + _args, _kwargs)
        return val
    def postcompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_postcompute,(self,) + _args, _kwargs)
        return val
    def residual(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_residual,(self,) + _args, _kwargs)
        return val
    def __del__(self,solverstepperc=solverstepperc):
        if self.thisown == 1 :
            solverstepperc.delete_SolverStepperCore(self)
    def __repr__(self):
        return "<C SolverStepperCore instance at %s>" % (self.this,)
class SolverStepperCore(SolverStepperCorePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(solverstepperc.new_SolverStepperCore,_args,_kwargs)
        self.thisown = 1




class NewtonSolverCorePtr(SolverStepperCorePtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def precompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_precompute,(self,) + _args, _kwargs)
        return val
    def solve(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_solve,(self,) + _args, _kwargs)
        return val
    def postcompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_postcompute,(self,) + _args, _kwargs)
        return val
    def residual(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_residual,(self,) + _args, _kwargs)
        return val
    def __del__(self,solverstepperc=solverstepperc):
        if self.thisown == 1 :
            solverstepperc.delete_NewtonSolverCore(self)
    def __repr__(self):
        return "<C NewtonSolverCore instance at %s>" % (self.this,)
class NewtonSolverCore(NewtonSolverCorePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(solverstepperc.new_NewtonSolverCore,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------



#-------------- USER INCLUDE -----------------------

# -*- python -*-
# $RCSfile: solverstepper_.py,v $
# $Revision: 1.35 $
# $Author: redwing $
# $Date: 2004/09/23 14:56:03 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import debug
from common import cregisteredclass
from common import registeredclass
from common.IO import parameter
from common.IO import reporter
from common.IO import whoville
from SWIG.common import switchboard
from SWIG.common import devel
from SWIG.common.IO import progressbar
from SWIG.engine import ooferror2
from SWIG.engine import solver

import engine.mesh
import time

##
## Iterator Managers for SolverDrivers start HERE
##
## A SolverIteratorManager is a class that keeps
## track of the state of the iterations and the
## used solver. If the linear solver used fails
## or the maximum number of iterations is reached
## the SolverIterationManager raises a flag that
## the is used by the SolverDriver to bail-out of
## the loop.

class SolverIterationManager : ## (registeredclass.RegisteredClass):
    registry = []
    def __init__(self, num_steps):
        self.max_num_steps = num_steps
        self.continew = 1 ## flag to stop the iterator from doing stuff 1==keep rocking, 2==go catch the NIST shuttle, 
        self.counter = 0 ## number of steps
        
    def update(self):
        ## update is meant to check the state of the
        ## iterator at the end of each step and
        ## set the most recent value for the state
        ## of the control variables
        pbar = progressbar.getcProgressBar()
        self.counter +=1
        if self.counter >= self.max_num_steps \
               or pbar.query_stop() !=0 \
               or pbar.get_success() is progressbar.FAILED:
            ## If the maximum number of iterations is reached
            ## or somewhere inside the stop button was pushed
            ## or if the state of the progress bar reports fail
            ## then break-out of the loop
            self.continew = 0 ## do not continue state
            pbar.set_message("Failed")
        else:
            ## otherwise, update the state of the progress bar
            ## to reflect the current iteration.
            pbar.set_progress(1.0*self.counter/self.max_num_steps)
            pbar.set_message("%d/%d"%(self.counter, self.max_num_steps))
            
    def checkpoint(self):
        ## return current state of the solver
        return self.continew
    def set_residual(self, residual):
        ## set residual of applied solver
        self.residual = residual
    def get_residual (self):
        ## return value of residual
        return residual

## The NewtonIterationManager is a special type of Iteration
## manager that adds extra conditionals to the base
## SolverIterationManager class. Here, the residual of the
## calculation is being tracked
class NewtonIterationManager (SolverIterationManager):
    def __init__(self, num_steps, tolerance):
        SolverIterationManager.__init__(self, num_steps)
        self.tolerance = tolerance
        self.residual = 100000.0 ## residual is updated on every pass
    def update(self):
        pbar = progressbar.getcProgressBar()
        SolverIterationManager.update(self) ## first check previous flags
        if self.residual <= self.tolerance:
            ## if residual is smaller than specified tolerance
            ## then stop iterating and say that you have succeeded
            self.continew =0
            pbar.set_message("Succeded")
            
    
## SolverDriver is the base class for all the Solver Iterators/Steppers
## that use a linear solver as an element of the iteration process.
##
## The API is such that self.apply() is the function that it is called
## to make the SolverDrivers work.
class SolverDriver (registeredclass.RegisteredClass) :
    registry = []
    def __init__(self, method, mesh, criterion):
        ## mesh is a who object
        self.method = method ## corresponds to the linear solver to be used
        self.mesh = mesh ## a who object
        self.core = SolverStepperCore(self.method, mesh.getObject())
        self.criterion = criterion ## a SolverIteratorManager object
        self.status = None ## state of the linear solver at each step
    def update_properties(self):
        ## currently unused function
        ## It is meant to update non-linear, associated fields.
        ## Potential examples include remanent elasticity, polarization
        ## and magnetization.
        if not self.criterion.checkpoint():
            return
        ## expand floating BCs
        if self.status and self.criterion.checkpoint():
            self.mesh.getObject().expand_float_bcs()
            ## reporter.report("Expanded floating BC's, if any.")
            
        self.mesh.getObject().precompute() ## gets materials from pixel groups
        reporter.start_progressbar()
        self.mesh.getObject().make_stiffness() ## calculates stiffness matrix
        reporter.progressbar_completed()
        reporter.end_progressbar()
        self.core.precompute() ## recalculates the A, rhs, and x vectors
    def iterate(self):
        ## called by self.solve()
        ##
        ## This is the main loop where properties and solvers interact
        ## with each other to converge to THE solution
        ##
        while self.criterion.checkpoint():
            self.equilibrate() ## calls the solver  and starts pbars
            self.criterion.set_residual(self.core.residual()) ## updates residual value in iterator
            ## print "residual=", self.criterion.residual
            self.criterion.update() ## updates criterion for bailing-out of loop
    def precompute(self):
        ## 
        ## Here, the finite element mesh is built: stiffnes matrix,
        ## boundary conditions are assigned, floating boundary nodes
        ## are promoted, dofs are mapped, and the actual A, x, and b objects
        ## are created (for solving the A x = b)
        ##
        ## first check for quality of materials
        badmatls = self.mesh.getObject().check_materials()
        if badmatls:
            msgs = []
            for matl in badmatls:
                msgs += matl.consistency_messages()
            raise ooferror2.ErrUserError('Badly defined materials!\n' +
                                         string.join(msgs, '\n'))
        ## grab the associated progressbar
        eq_cProgressBar = progressbar.getcProgressBar()

        # precompute must be called before anything else that
        # might want to check if a Property is active.
        self.mesh.getObject().precompute() ## precomputes materials
        reporter.start_progressbar()
        # smtstart = time.time()
        self.mesh.getObject().make_stiffness() ## makes stiffness matrix
        # smtend = time.time()
        reporter.progressbar_completed()
        reporter.end_progressbar()

    ## Take measures if process failed
        if eq_cProgressBar.query_stop() !=0:
            eq_cProgressBar.set_failure()
            eq_cProgressBar.set_progress(1)
            eq_cProgressBar.set_message("Interrupted") ## GUI message
            reporter.progressbar_cleanup()
            reporter.warn("Process interrupted")
            return
    
        
        # Initialize the vector that records the boundary's contribution
        # to the RHS.  Correct size, full of zeros.
        self.mesh.getObject().boundary_rhs_init( self.mesh.getObject().stiffness_matrix().nrows() )
        reporter.report("Made stiffness matrix")
        # reporter.report("Construction time: %f" % (smtend-smtstart))
        
        # Make sure all dofs and equations are initially free/independent.
        self.mesh.getObject().resetdofeqs()
        
        ## Resolve intersecting float/fixed BC's, and check fixed BC's.
        ##
        ## Note to developers:
        ## 
        ## Intersecting BC means that more than one boundary condition
        ## is specified at the boundary nodes (i.e., the intersection of
        ## two sets of nodes with user-specified BCs is not null, but
        ## not necessarily redundant).
        ##
        ## A floating boundary condition is one where only the difference
        ## of the values between neighboring nodes is specified. The
        ## field value is not fixed at the node, except when the node
        ## intersects a BC which is fixed (in which case it is promoted).
        ##
        ## A fixed BC is that one where the nodal value of the field
        ## is specified.
        ##
        ## A promoted BC node/dof is a node that is changed in status
        ## from floating to fixed. 
        while self.mesh.getObject().bcPromote():
            pass
        self.mesh.getObject().fixedBCConsistency()
        reporter.report("Completed Fixed BC consistency checks.")
        
        self.mesh.getObject().floatBCResolve()
        reporter.report("Resolved floating BC intersections, if any.")
        
        self.mesh.getObject().invoke_flux_bcs()
        reporter.report("Applied flux boundary conditions, if any.")
        self.mesh.getObject().invoke_fixed_bcs()
        reporter.report("Applied fixed boundary conditions, if any.")
        self.mesh.getObject().invoke_force_bcs()
        reporter.report("Applied force boundary conditions, if any.")

        self.mesh.getObject().mapdofeqs()      # must come after fixed bcs, before floating ones

    
        reporter.report("Mapped equations.")
        self.mesh.getObject().invoke_float_bcs()
        reporter.report("Applied floating boundary conditions, if any.")
        ## the next line is the very last thing that must be done
        ## before actually solving the mesh
        self.core.precompute() ## allocates memory and builds the actual
        ## A-matrix, b-vector, x-vector to solve the system Ax=b
        ## debug.fmsg("after precomputing the core")
    def postcompute(self):
        ##
        ## Here, floating BCs are expanded, reset,
        ## and history-dependent fields are fixated
        ##
        eq_cProgressBar = progressbar.getcProgressBar()
        if self.status:
            self.mesh.getObject().expand_float_bcs()
            reporter.report("Expanded floating BC's, if any.")
            self.mesh.solutiontimestamp.increment() # Increment the right timestamp

            # Earliest opportunity to remove automatically-generated fixed
            # BCs caused by promotion.
        self.mesh.getObject().reset_bcs() ## not too expensive

        reporter.start_progressbar()
        self.mesh.getObject().post_process() ## plasticity fixation and memory effects are set here
        reporter.end_progressbar()
        
        reporter.report("Completed post-processing, if any.")
        reporter.progressbar_cleanup()
        if not eq_cProgressBar.get_success() or \
               not eq_cProgressBar.query_stop():
            info = self.method.message()
            reporter.report(info)
        status = eq_cProgressBar.get_success()
        message = self.method.message()
        if status == progressbar.FAILED:
            if eq_cProgressBar.query_stop():
                self.mesh.setSolverStatus('interrupted', self.method, message)
            else:
                self.mesh.setSolverStatus('failed', self.method, message)
        elif status == progressbar.INCOMPLETE:
            self.mesh.setSolverStatus('failed', self.method, 'setup failed')
        elif status == progressbar.COMPLETE:
            self.mesh.setSolverStatus('completed', self.method, message)
        switchboard.notify('mesh solved', self.mesh)
        
    def postcompute_and_redraw(self):
        ## does as advertised
        self.postcompute()
        self.redraw()
    def redraw(self):
        switchboard.notify('redraw')
    def solve(self):
        ## is this okay for linear solvers??
        self.precompute()
        self.iterate()
        self.postcompute_and_redraw()
    def equilibrate(self):
        reporter.start_progressbar()
        # solvestart = time.time()
        self.status = self.core.solve()
        # solveend = time.time()
        reporter.progressbar_completed()
        reporter.end_progressbar()

    
## LinearSolver is a single-step linear solver.
## It is basically a wrapper for the solver(s).
## It is meant to provide a uniform interface to the
## linear solver page.

## TODO LATER: Make the Solvers aware of the equations that
## they should (and can) solve.
        
class LinearSolver (SolverDriver):
    def __init__(self, method):
        self.method = method
    def apply(self, meshcontext):
        from SWIG.engine import solverstepper
        equilibrate = solverstepper.SolverDriver(
            self.method, meshcontext, solverstepper.SolverIterationManager(1))
        equilibrate.precompute()
        equilibrate.iterate()
        equilibrate.postcompute_and_redraw()

registeredclass.Registration(
    'LinearSolver',
    SolverDriver, ## superclass
    LinearSolver, ## subclass
    ordering = 100,
    params = [
    parameter.Parameter('method', solver.SolverPtr)
    ]
    )


##
## NewtonSolver is a Newton-Raphson iterator.
## It is meant to provide a general framework for
## the solution of time-independent, field-dependent
## problems.
##
##
class NewtonSolver (SolverDriver):
    def __init__(self, method, newton_tolerance, newton_max_iterations):
        ## mesh is a who object
        self.method = method
        self.mesh = None
        self.core = None
        self.newton_tolerance = newton_tolerance
        self.newton_max_iterations = newton_max_iterations
        self.criterion = NewtonIterationManager(self.newton_max_iterations, self.newton_tolerance)
        self.status = None ## state of the linear solver at each step
    def iterate(self):
        while self.criterion.checkpoint():
            self.precompute()
            self.equilibrate()
            self.postcompute()
            self.criterion.set_residual(self.core.residual()) ## updates residual value in iterator
            ## print "residual=", self.criterion.residual
            self.criterion.update()
            self.mesh.solutiontimestamp.increment()
            ## debug.fmsg("incremented timestamp")


            ## debug.fmsg("counter=", self.criterion.counter)

    def apply(self, meshcontext):
        ## debug.fmsg("entering NR.apply()")
        try:
            self.mesh = meshcontext
            self.core = NewtonSolverCore(self.method, meshcontext.getObject())
            
            self.iterate() ## iterate updates properties at the end of each cycle
        finally:
            self.redraw()
            
    def apply_realmesh(self, mesh):
        ## debug.fmsg("entering NR.apply()")
        try:
            self.mesh = mesh
            self.core = NewtonSolverCore(self.method, mesh) ## ????
            
            self.iterate() ## iterate updates properties at the end of each cycle
        finally:
            self.redraw()

if devel.devel():
    registeredclass.Registration(
        'NewtonRaphson',
        SolverDriver,
        NewtonSolver,
        ordering=200,
        params=[
        parameter.Parameter('method', solver.SolverPtr,
                            tip='linear solver is used by NR method to find non-linear solution'),
        parameter.FloatParameter('newton_tolerance', 1e-8,
                                 tip='convergence criteria'),
        parameter.IntParameter('newton_max_iterations', 10,
                               tip='maximum number of Newton steps before bailing-out')
        
        ],
        tip='Newton-Raphson solver for the solution of field-dependent material properties'
        )
