# -*- python -*-
# $RCSfile: symmmatrix_.py,v $
# $Revision: 1.18 $
# $Author: reida $
# $Date: 2004/08/24 14:55:14 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

#
# __setitem__/__getitem__ overrides allow this to work like a
# dictionary.  See also cijkl_.py in the elasticity property files.
#

from SWIG.engine import fieldindex
from common import debug
from common import enum
from common import utils
from common import registeredclass
from common.IO import parameter
from common.IO import output
import string
import struct
import types

def _SymmMatrix___setitem__(self,idx,val):
    size = self.size()
    if len(idx)==2 and 0<=idx[0]<size and 0<=idx[1]<size:
        self.set(idx[0],idx[1],val)
    else:
        raise KeyError("Index out of bounds in SymmMatrix")

SymmMatrixPtr.__setitem__ = _SymmMatrix___setitem__

def _SymmMatrix___getitem__(self,idx):
    size = self.size()
    if len(idx)==2 and 0<=idx[0]<size and 0<=idx[1]<size:
        return self.get(idx[0],idx[1])
    else:
        raise KeyError("Index out of bounds in SymmMatrix")

SymmMatrixPtr.__getitem__ = _SymmMatrix___getitem__

# Mathematica-compatible repr, for debugging.
def _SymmMatrix___repr__(self):
    rowlist = []
    for i in range(self.size()):
        collist = []
        for j in range(self.size()):
            collist.append("%f" % self.get(i,j))
        rowlist.append("{ " + string.join(collist,", ") + " }")
    return "SymmMatrix { " + string.join(rowlist,", ") + " }"

SymmMatrixPtr.__repr__ = _SymmMatrix___repr__

################################################
################################################

# The 3x3 Symmetric Matrix gets special treatment

# Redefine the init so that it accepts keyword arguments (swigged
# functions don't allow keywords).  Have unspecified entries default
# to zero.

_oldSymmMatrix3___init__ = SymmMatrix3.__init__
def _SymmMatrix3___init__(self, xx=0.0, yy=0.0, zz=0.0,
                          yz=0.0, xz=0.0, xy=0.0):
    _oldSymmMatrix3___init__(self, xx, yy, zz, yz, xz, xy)
SymmMatrix3.__init__ = _SymmMatrix3___init__
    
voigtIndices = ((0,0), (1,1), (2,2), (1,2), (0,2), (0,1))
voigtNames = ('xx', 'yy', 'zz', 'yz', 'xz', 'xy')


class VoigtComponent(enum.EnumClass(*voigtNames)): pass

def _SymmMatrix3_voigtValues(self):
    return [self.get(*voigtIndices[i]) for i in range(6)]

SymmMatrix3.voigtValues = _SymmMatrix3_voigtValues

def _SymmMatrix3___repr__(self):
    args = ["%s=%f" % nv for nv in zip(voigtNames, self.voigtValues())]
    return "SymmMatrix3(%s)" % string.join(args, ',')

SymmMatrix3.__repr__ = _SymmMatrix3___repr__

def _SymmMatrix3_component(self, component): # used in Output, below
    # component is a VoigtComponent Enum object
    return self.get(*voigtIndices[component.index()])
SymmMatrix3Ptr.component = _SymmMatrix3_component

utils.OOFdefine('SymmMatrix3', SymmMatrix3)

############

class SymmMatrix3Invariant(registeredclass.RegisteredClass):
    registry = []
    # SymmMatrix3Invariant subclasses must have a __call__ method
    # which takes a SymmMatrix3 and returns a Float.

class STrace(SymmMatrix3Invariant):
    def __call__(self, matrix):
        return matrix.trace()

registeredclass.Registration('Trace', SymmMatrix3Invariant, STrace,
                             ordering=0)

class SSecondInvariant(SymmMatrix3Invariant):
    def __call__(self, matrix):
        return matrix.secondInvariant()

registeredclass.Registration('Second Invariant', SymmMatrix3Invariant,
                             SSecondInvariant, ordering=1)

class SDeterminant(SymmMatrix3Invariant):
    def __call__(self, matrix):
        return matrix.determinant()

registeredclass.Registration('Determinant', SymmMatrix3Invariant,
                             SDeterminant, ordering=2)

############

class SymmMatrix3Parameter(parameter.ParameterBase):
    def checker(self, x):
        if x is not None and not isinstance(x, SymmMatrix3Ptr):
            parameter.raiseTypeError(type(x), SymmMatrix3)
    structfmt =">6d"
    structsize = struct.calcsize(structfmt)
    def binaryRepr(self, datafile, value):
        return struct.pack(SymmMatrix3Parameter.structfmt, *value.voigtValues())
    def binaryRead(self, parser):
        b = parser.getBytes(SymmMatrix3Parameter.structsize)
        vv = struct.unpack(SymmMatrix3Parameter.structfmt, b)
        return SymmMatrix3(*vv)

### Outputs for SymmMatrix3s.

##def _SymmMatrixComponent(sm3s, parameters, *args, **kwargs):
##    comp = parameters[0].value          # A VoigtComponent
##    return [sm3.component(comp) for sm3 in sm3s]

##SymmMatrix3ComponentOutput = output.Output(
##    name="SymmMatrix3 component",
##    func=_SymmMatrixComponent,
##    oType=types.FloatType,
##    inputs=output.Input("symmmatrix3", SymmMatrix3),
##    parameters=enum.EnumParameter("component", VoigtComponent, default="xx"))


##def _SymmMatrixInvariant(sm3s, parameters, *args, **kwargs):
##    invariant = parameters[0].value     # A SymmMatrix3Invariant
##    return map(invariant, sm3s)

##SymmMatrix3InvariantOutput = output.Output(
##    name="SymmMatrix3 invariant",
##    func=_SymmMatrixInvariant,
##    oType=types.FloatType,
##    inputs=output.Input("symmmatrix3", SymmMatrix3),
##    parameters=[parameter.RegisteredParameter("invariant", SymmMatrix3Invariant,
##                                              default=STrace())])
